#-----------------------------------------------------------------------------
# Group to Morph altered for Explosion:
# Main change is that the verts move with the facets, using the
# Polynorm method as in Exploder. I took out the straight/round/ridge
# choice; the only choice is the explode-type move.

# Sep. 20, 2004
# Copyright (c) 2004 by David G. Drumright (ockham)

#-----------------------------------------------------------------------------

import poser
import string
import math
from Tkinter import *
scene = poser.Scene()

#-----------------------------------------------------------------------------
# Global vars
#-----------------------------------------------------------------------------

global BaseActor
global BaseGeom
global SelectedGroups
global DeltaVerts
global GroupedFacets
global SortedVerts
global CenterX
global CenterY
global CenterZ

BaseActor=0
BaseGeom=0
GroupedFacets=[]
SelectedGroups={} # index is number in list, content is name.
SortedVerts=[]
DeltaVerts={} # Runs parallel to SortedVerts

# These are set up as dicts so we can
# make non-contiguous arrays.

CenterX=0.0
CenterY=0.0
CenterZ=0.0
CoarseInc = 0.001
FineInc = 0.0001

#-----------------------------------------------------------------------------

def InitFirst():
    # Check the basics before putting anything up.
    global BaseActor
    global BaseGeom

    BaseActor = scene.CurrentActor()  # There's always a CurrentActor
    try:
        # Get the Geom of the chosen actor:
        BaseGeom = BaseActor.Geometry()
    except:
        raise "Chosen part doesn't have a mesh."

    # Get the group list so we can let you select.
    # You may want to use one of the pre-existing
    # groups, or you may have made your own.

    if BaseGeom.Groups() == None:
        raise "No groups in this actor"

#-----------------------------------------------------------------------------
def PolyNorm(PList):
    # Takes a list holding first points of polygon as tuples, gives normal of poly in cos values.
    if len(PList) < 3: return (0.0,0.0,0.0)
    (P0x,P0y,P0z)=PList[0]
    (P1x,P1y,P1z)=PList[1]
    (P2x,P2y,P2z)=PList[2]
    # Use only first three points.


    A = (P2y - P0y)*(P1z - P0z) - (P1y - P0y)*(P2z - P0z)
    B = (P1x - P0x)*(P2z - P0z) - (P2x - P0x)*(P1z - P0z)
    C = (P2x - P0x)*(P1y - P0y) - (P1x - P0x)*(P2y - P0y)
    Dist= math.sqrt(A*A+B*B+C*C)
    if Dist == 0.0:
        return (0.0,0.0,0.0)
    else:
        Ax = A/Dist
        By = -B/Dist
        Cz = C/Dist
        return (Ax,By,Cz)

#-----------------------------------------------------------------------------

def LocateAllVerts():
    # This part of the init is called AFTER you select some groups.
    global GroupedFacets
    global PolyNorms
    global SortedVerts
    global DeltaVerts
    global CenterX
    global CenterY
    global CenterZ

    # Start the Facets and Verts lists with just the basics,
    # and the default markings for Group and Layer.
    # and the default markings for Group and Layer.
    # Reminder: use only the group-list index, not the group name,
    # in places where group needs to be an index.
    group = -1    # default index: real one will be >=0
    dist  = -1.0  # default net distance: real will be >=0.0
    X = 0.0
    Y = 0.0
    Z = 0.0

    # Go through the Polygons (facets) of the Geom, by Groups.
    # Put all verts around the grouped facets into the SortedVerts array.
    # Note that this forcibly assumes split verts!

    RawVerts=[]  # This is a list, not a dict.  Nec for Ridge, to get dups

    SetList = BaseGeom.Sets()
    for GroupIndex in SelectedGroups.keys():
        GroupName = SelectedGroups[GroupIndex] # optimize
        for FacetIndex in range(BaseGeom.NumPolygons()):
            OneFacet = BaseGeom.Polygon(FacetIndex)
            if OneFacet.InGroup(GroupName):
                GroupedFacets.append(FacetIndex)
                First = OneFacet.Start()
                Last = First + OneFacet.NumVertices()
                for V in SetList[First:Last]: # This is a list of vertex indexes in the poly
                    # Get the values:
                    OneVert = BaseGeom.Vertex(V)
                    X = OneVert.X()
                    Y = OneVert.Y()
                    Z = OneVert.Z()
                    # Fill in the Group and XYZ info.
                    RawVerts.append((V,GroupIndex,dist,X,Y,Z))
                    # Note that this can include the same V more than once,
                    # if that V was in more than one group.

    # Now set up the SortedVerts and DeltaVerts arrays.
    SumX=0.0
    SumY=0.0
    SumZ=0.0
    Count=0.0
    for i in range(len(RawVerts)):
         V,G,D,X,Y,Z = RawVerts[i]
         if G >= 0:  # Leave out the ungrouped
             SortedVerts.append(V)
             DeltaVerts[V] = V,G,D,0.0,0.0,0.0
             # Form group center for use in Contract/Expand
             SumX = SumX + X
             SumY = SumY + Y
             SumZ = SumZ + Z
             Count=Count+1.0
    if Count > 0.0:
        CenterX=SumX/Count
        CenterY=SumY/Count
        CenterZ=SumZ/Count
    else:
        CenterX=0.0
        CenterY=0.0
        CenterZ=0.0

#-----------------------------------------------------------------------------
# The Displace routines actually move the verts so you can see what's
# going on.  They also change all verts in the ChangedVerts array.
#-----------------------------------------------------------------------------

def DisplaceX(delta):
    global DeltaVerts
    for V in DeltaVerts.keys():
        OneVert = BaseGeom.Vertex(V)
        (dummyV,G,D,OrigX,OrigY,OrigZ) = DeltaVerts[V]
        OneVert.SetX(OneVert.X()+delta)
        DeltaVerts[V] = (dummyV,G,D,delta,OrigY,OrigZ)

#-----------------------------------------------------------------------------

def DisplaceY(delta):
    global DeltaVerts
    for V in DeltaVerts.keys():
        OneVert = BaseGeom.Vertex(V)
        (dummyV,G,D,OrigX,OrigY,OrigZ) = DeltaVerts[V]
        OneVert.SetY(OneVert.Y()+delta)
        DeltaVerts[V] = (dummyV,G,D,OrigX,delta,OrigZ)

#-----------------------------------------------------------------------------

def DisplaceZ(delta):
    global DeltaVerts
    for V in DeltaVerts.keys():
        OneVert = BaseGeom.Vertex(V)
        dummyV,G,D,OrigX,OrigY,OrigZ = DeltaVerts[V]
        OneVert.SetZ(OneVert.Z()+delta)
        DeltaVerts[V] = dummyV,G,D,OrigX,OrigY,delta

#-----------------------------------------------------------------------------

def DisplaceBulge(delta):
    global DeltaVerts
    Norms = BaseGeom.Normals()
    for V in DeltaVerts.keys():
        OneVert = BaseGeom.Vertex(V)
        (dummyV,G,D,OrigX,OrigY,OrigZ) = DeltaVerts[V]
        dX= Norms[V].X() * delta
        dY= Norms[V].Y() * delta
        dZ= Norms[V].Z() * delta
        OneVert.SetX(OneVert.X()+dX)
        OneVert.SetY(OneVert.Y()+dY)
        OneVert.SetZ(OneVert.Z()+dZ)
        DeltaVerts[V] = (dummyV,G,D,dX,dY,dZ)

#-----------------------------------------------------------------------------

def DisplaceXscale(delta):
    global DeltaVerts
    for V in DeltaVerts.keys():
        OneVert = BaseGeom.Vertex(V)
        (dummyV,G,D,OrigX,OrigY,OrigZ) = DeltaVerts[V]
        # Find direction of this vert from Group center.
        # If this vert is farther from Group center, move it farther
        # in the same direction.
        dX=(OneVert.X()-CenterX ) * delta * 5.0
        OneVert.SetX(OneVert.X()+dX)
        DeltaVerts[V] = (dummyV,G,D,dX,OrigY,OrigZ)

#-----------------------------------------------------------------------------

def DisplaceYscale(delta):
    global DeltaVerts
    for V in DeltaVerts.keys():
        OneVert = BaseGeom.Vertex(V)
        (dummyV,G,D,OrigX,OrigY,OrigZ) = DeltaVerts[V]
        # Find direction of this vert from Group center.
        # If this vert is farther from Group center, move it farther
        # in the same direction.
        dY=(OneVert.Y()-CenterY ) * delta * 5.0
        OneVert.SetY(OneVert.Y()+dY)
        DeltaVerts[V] = (dummyV,G,D,OrigX,dY,OrigZ)

#-----------------------------------------------------------------------------

def DisplaceZscale(delta):
    global DeltaVerts
    for V in DeltaVerts.keys():
        OneVert = BaseGeom.Vertex(V)
        (dummyV,G,D,OrigX,OrigY,OrigZ) = DeltaVerts[V]
        # Find direction of this vert from Group center.
        # If this vert is farther from Group center, move it farther
        # in the same direction.
        dZ=(OneVert.Z()-CenterZ ) * delta * 5.0
        OneVert.SetZ(OneVert.Z()+dZ)
        DeltaVerts[V] = (dummyV,G,D,OrigX,OrigY,dZ)

#-----------------------------------------------------------------------------

def SpawnResult(Name):
    global DeltaVerts
    global BaseGeom
    DeltaX=0.0
    DeltaY=0.0
    DeltaZ=0.0

    fraction=0.02

    ArraySize = BaseGeom.NumVertices()
    NumDeltas = len(DeltaVerts)

    # Set all existing MTs to zero before starting.  Otherwise they will be
    # mysteriously affected by the new morph.  Reminder: don't do this
    # earlier; sometimes you need to have other morphs active so you
    # can see what's going on.
    for OneParm in BaseActor.Parameters():
        if OneParm.IsMorphTarget():
            OneParm.SetValue(0.0)
        scene.DrawAll()  # Show the 0 setting.

    if Name == "":
        Name = "GTM" # If you forgot a name, use this. Poser will add _1, etc auto.

    BaseActor.SpawnTarget(Name)  # Now create the new
    NewParm = BaseActor.Parameter(Name) # Fetch the new parm (Spawn doesn't return anything)

    PolyNormDict={}
    SetList = BaseGeom.Sets()
    # Run a loop by Polygons.
    for FacetIndex in GroupedFacets:
        OneFacet = BaseGeom.Polygon(FacetIndex)
        First = OneFacet.Start()
        Last = First + OneFacet.NumVertices()
        PolyNormList=[]
        for Vindex in SetList[First:Last]: # This is a list of vertex indexes in the poly
            # Get the values:
            OneVert = BaseGeom.Vertex(Vindex)
            X = OneVert.X()
            Y = OneVert.Y()
            Z = OneVert.Z()
            PolyNormList.append( (X,Y,Z) )
        # Now form the normal
        (NormX,NormY,NormZ) = PolyNorm(PolyNormList)
        PolyNormDict[FacetIndex]=(NormX,NormY,NormZ)

    SetList = BaseGeom.Sets()
    # Turn the explode motion into a MT.
    # Force all corners of each facet to move outward by the same proportion.
    for FacetIndex in GroupedFacets:
        (NormX,NormY,NormZ)=PolyNormDict[FacetIndex] # Get the worldnormals
        OneFacet = BaseGeom.Polygon(FacetIndex)
        First = OneFacet.Start()
        Last = First + OneFacet.NumVertices()
        for Vindex in SetList[First:Last]: # This is a list of vertex indexes in the poly
            OneVert = BaseGeom.Vertex(Vindex)
            DX = NormX * fraction
            DY = NormY * fraction
            DZ = NormZ * fraction
            NewParm.SetMorphTargetDelta(Vindex, -DX,DY,-DZ)

#------------------------------------------------------------
# Next section is TK
#------------------------------------------------------------

class App:
    def __init__(self, master, textMessage):
        self.master = master
        self.master.title("Selective Exploder")
        # Status goes on top
        self.StatusEntry=Entry(self.master,width=40)
        self.StatusEntry.grid(row=0,column=0,columnspan=3)
        self.StatusEntry.insert(0,"Select groups then hit Start.")

        # Frames for list and buttons
        self.ListFrame = Frame(self.master,borderwidth=2,relief=RIDGE)
        self.ListFrame.grid(row=1,column=0,pady=1)
        self.TranFrame = Frame(self.master,bg="beige",borderwidth=2,relief=RIDGE)
        self.TranFrame.grid( row=3,column=0,pady=1)
        self.ScaleFrame = Frame(self.master,bg="lightblue",borderwidth=2,relief=RIDGE)
        self.ScaleFrame.grid( row=4,column=0,pady=1)
        self.BulgeFrame = Frame(self.master,bg="pink",borderwidth=2,relief=RIDGE)
        self.BulgeFrame.grid( row=5,column=0,pady=1)
        self.BottomFrame = Frame(self.master,borderwidth=2,relief=RIDGE)
        self.BottomFrame.grid(row=6,column=0,pady=1)

        # List and scroll
        self.GroupScroll  = Scrollbar(self.ListFrame, orient=VERTICAL)
        self.GroupScroll.grid( row=0, column=0,sticky=N+S+E)
        self.GroupBox  = Listbox(self.ListFrame, height=5, selectmode=MULTIPLE,yscrollcommand = self.GroupScroll.set)
        self.GroupBox.grid( row=0, column=1)
        self.GroupStart = Button(self.ListFrame,  text="Start moves",command=self.HandleGroupStart)
        self.GroupStart.grid (row=1, column=0)

        # Fill with names of Groups
        for OneName in BaseGeom.Groups():
            self.GroupBox.insert(END,OneName)
        # Set the scrollbar to work
        self.GroupScroll["command"] = self.GroupBox.yview

        # Coarse and fine buttons
        self.XtranLeftCoarse  = Button(self.TranFrame,text="<<",command=self.HandleXtranLeftCoarse,state=DISABLED)
        self.XtranLeftFine    = Button(self.TranFrame,text="<" ,command=self.HandleXtranLeftFine,state=DISABLED)
        self.XtranLabel       = Label (self.TranFrame,text="X tran",bg="beige")
        self.XtranRightFine   = Button(self.TranFrame,text=">" ,command=self.HandleXtranRightFine,state=DISABLED)
        self.XtranRightCoarse = Button(self.TranFrame,text=">>",command=self.HandleXtranRightCoarse,state=DISABLED)
        self.XtranLeftCoarse.grid (row=0, column=0)
        self.XtranLeftFine.grid   (row=0, column=1)
        self.XtranLabel.grid      (row=0, column=2)
        self.XtranRightFine.grid  (row=0, column=3)
        self.XtranRightCoarse.grid(row=0, column=4)

        self.YtranLeftCoarse  = Button(self.TranFrame,text="<<",command=self.HandleYtranLeftCoarse,state=DISABLED)
        self.YtranLeftFine    = Button(self.TranFrame,text="<" ,command=self.HandleYtranLeftFine,state=DISABLED)
        self.YtranLabel       = Label (self.TranFrame,text="Y tran",bg="beige")
        self.YtranRightFine   = Button(self.TranFrame,text=">" ,command=self.HandleYtranRightFine,state=DISABLED)
        self.YtranRightCoarse = Button(self.TranFrame,text=">>",command=self.HandleYtranRightCoarse,state=DISABLED)
        self.YtranLeftCoarse.grid (row=1, column=0)
        self.YtranLeftFine.grid   (row=1, column=1)
        self.YtranLabel.grid      (row=1, column=2)
        self.YtranRightFine.grid  (row=1, column=3)
        self.YtranRightCoarse.grid(row=1, column=4)

        self.ZtranLeftCoarse  = Button(self.TranFrame,text="<<",command=self.HandleZtranLeftCoarse,state=DISABLED)
        self.ZtranLeftFine    = Button(self.TranFrame,text="<" ,command=self.HandleZtranLeftFine,state=DISABLED)
        self.ZtranLabel       = Label (self.TranFrame,text="Z tran",bg="beige")
        self.ZtranRightFine   = Button(self.TranFrame,text=">" ,command=self.HandleZtranRightFine,state=DISABLED)
        self.ZtranRightCoarse = Button(self.TranFrame,text=">>",command=self.HandleZtranRightCoarse,state=DISABLED)
        self.ZtranLeftCoarse.grid (row=2, column=0)
        self.ZtranLeftFine.grid   (row=2, column=1)
        self.ZtranLabel.grid      (row=2, column=2)
        self.ZtranRightFine.grid  (row=2, column=3)
        self.ZtranRightCoarse.grid(row=2, column=4)


        self.XscaleLeftCoarse  = Button(self.ScaleFrame,text="<<",command=self.HandleXscaleLeftCoarse,state=DISABLED)
        self.XscaleLeftFine    = Button(self.ScaleFrame,text="<" ,command=self.HandleXscaleLeftFine,state=DISABLED)
        self.XscaleLabel       = Label (self.ScaleFrame,text="X scale",bg="lightblue")
        self.XscaleRightFine   = Button(self.ScaleFrame,text=">" ,command=self.HandleXscaleRightFine,state=DISABLED)
        self.XscaleRightCoarse = Button(self.ScaleFrame,text=">>",command=self.HandleXscaleRightCoarse,state=DISABLED)
        self.XscaleLeftCoarse.grid (row=0, column=0)
        self.XscaleLeftFine.grid   (row=0, column=1)
        self.XscaleLabel.grid      (row=0, column=2)
        self.XscaleRightFine.grid  (row=0, column=3)
        self.XscaleRightCoarse.grid(row=0, column=4)

        self.YscaleLeftCoarse  = Button(self.ScaleFrame,text="<<",command=self.HandleYscaleLeftCoarse,state=DISABLED)
        self.YscaleLeftFine    = Button(self.ScaleFrame,text="<" ,command=self.HandleYscaleLeftFine,state=DISABLED)
        self.YscaleLabel       = Label (self.ScaleFrame,text="Y scale",bg="lightblue")
        self.YscaleRightFine   = Button(self.ScaleFrame,text=">" ,command=self.HandleYscaleRightFine,state=DISABLED)
        self.YscaleRightCoarse = Button(self.ScaleFrame,text=">>",command=self.HandleYscaleRightCoarse,state=DISABLED)
        self.YscaleLeftCoarse.grid (row=1, column=0)
        self.YscaleLeftFine.grid   (row=1, column=1)
        self.YscaleLabel.grid      (row=1, column=2)
        self.YscaleRightFine.grid  (row=1, column=3)
        self.YscaleRightCoarse.grid(row=1, column=4)

        self.ZscaleLeftCoarse  = Button(self.ScaleFrame,text="<<",command=self.HandleZscaleLeftCoarse,state=DISABLED)
        self.ZscaleLeftFine    = Button(self.ScaleFrame,text="<" ,command=self.HandleZscaleLeftFine,state=DISABLED)
        self.ZscaleLabel       = Label (self.ScaleFrame,text="Z scale",bg="lightblue")
        self.ZscaleRightFine   = Button(self.ScaleFrame,text=">" ,command=self.HandleZscaleRightFine,state=DISABLED)
        self.ZscaleRightCoarse = Button(self.ScaleFrame,text=">>",command=self.HandleZscaleRightCoarse,state=DISABLED)
        self.ZscaleLeftCoarse.grid (row=2, column=0)
        self.ZscaleLeftFine.grid   (row=2, column=1)
        self.ZscaleLabel.grid      (row=2, column=2)
        self.ZscaleRightFine.grid  (row=2, column=3)
        self.ZscaleRightCoarse.grid(row=2, column=4)

        self.BulgeLeftCoarse  = Button(self.BulgeFrame,text="<<",command=self.HandleBulgeLeftCoarse,state=DISABLED)
        self.BulgeLeftFine    = Button(self.BulgeFrame,text="<" ,command=self.HandleBulgeLeftFine,state=DISABLED)
        self.BulgeLabel       = Label (self.BulgeFrame,text=" Bulge",bg="pink")
        self.BulgeRightFine   = Button(self.BulgeFrame,text=">" ,command=self.HandleBulgeRightFine,state=DISABLED)
        self.BulgeRightCoarse = Button(self.BulgeFrame,text=">>",command=self.HandleBulgeRightCoarse,state=DISABLED)
        self.BulgeLeftCoarse.grid (row=0, column=0)
        self.BulgeLeftFine.grid   (row=0, column=1)
        self.BulgeLabel.grid      (row=0, column=2)
        self.BulgeRightFine.grid  (row=0, column=3)
        self.BulgeRightCoarse.grid(row=0, column=4)

        # Name and Go buttons, in Bottom frame
        self.NameLabel = Label(self.BottomFrame,text="Name for new MT:")
        self.NameLabel.grid(row=0, column=0)
        self.NameEntry = Entry(self.BottomFrame,state=DISABLED)
        self.NameEntry.grid(row=0, column=1)
        self.buttonNormGo = Button(self.BottomFrame, text="Go", command=self.HandleNormGo,state=DISABLED)
        self.buttonNormGo.grid(row=1, column=0)
        self.buttonCancel = Button(self.BottomFrame, text="Cancel", command=self.HandleCancel)
        self.buttonCancel.grid(row=1, column=2)

        self.master.protocol('WM_DELETE_WINDOW', self.HandleCancel)

    # - - - - - - - - - - - - - - - - - -

    def ShowStatus(self,S): # Just saving repetition
        self.StatusEntry.delete(0,END)
        self.StatusEntry.insert(0,S)
        self.StatusEntry.update_idletasks()

    # - - - - - - - - - - - - - - - - - -

    def Update(self):
        scene.ProcessSomeEvents(1)
        root.lift()
        root.after(100, self.Update)

    # - - - - - - - - - - - - - - - - - -
    def HandleXtranLeftCoarse(self):
        DisplaceX(-CoarseInc)
        scene.DrawAll()
    # - - - - - - - - - - - - - - - - - -
    def HandleXtranLeftFine(self):
        DisplaceX(-FineInc)
        scene.DrawAll()
    # - - - - - - - - - - - - - - - - - -
    def HandleXtranRightFine(self):
        DisplaceX(+FineInc)
        scene.DrawAll()
    # - - - - - - - - - - - - - - - - - -
    def HandleXtranRightCoarse(self):
        DisplaceX(+CoarseInc)
        scene.DrawAll()
    # - - - - - - - - - - - - - - - - - -
    # - - - - - - - - - - - - - - - - - -
    def HandleYtranLeftCoarse(self):
        DisplaceY(-CoarseInc)
        scene.DrawAll()
    # - - - - - - - - - - - - - - - - - -
    def HandleYtranLeftFine(self):
        DisplaceY(-FineInc)
        scene.DrawAll()
    # - - - - - - - - - - - - - - - - - -
    def HandleYtranRightFine(self):
        DisplaceY(+FineInc)
        scene.DrawAll()
    # - - - - - - - - - - - - - - - - - -
    def HandleYtranRightCoarse(self):
        DisplaceY(+CoarseInc)
        scene.DrawAll()
    # - - - - - - - - - - - - - - - - - -
    # - - - - - - - - - - - - - - - - - -
    def HandleZtranLeftCoarse(self):
        DisplaceZ(-CoarseInc)
        scene.DrawAll()
    # - - - - - - - - - - - - - - - - - -
    def HandleZtranLeftFine(self):
        DisplaceZ(-FineInc)
        scene.DrawAll()
    # - - - - - - - - - - - - - - - - - -
    def HandleZtranRightFine(self):
        DisplaceZ(+FineInc)
        scene.DrawAll()
    # - - - - - - - - - - - - - - - - - -
    def HandleZtranRightCoarse(self):
        DisplaceZ(+CoarseInc)
        scene.DrawAll()
    # - - - - - - - - - - - - - - - - - -
    # - - - - - - - - - - - - - - - - - -
    def HandleBulgeLeftCoarse(self):
        DisplaceBulge(-CoarseInc)
        scene.DrawAll()
    # - - - - - - - - - - - - - - - - - -
    def HandleBulgeLeftFine(self):
        DisplaceBulge(-FineInc)
        scene.DrawAll()
    # - - - - - - - - - - - - - - - - - -
    def HandleBulgeRightFine(self):
        DisplaceBulge(+FineInc)
        scene.DrawAll()
    # - - - - - - - - - - - - - - - - - -
    def HandleBulgeRightCoarse(self):
        DisplaceBulge(+CoarseInc)
        scene.DrawAll()
    # - - - - - - - - - - - - - - - - - -
    # - - - - - - - - - - - - - - - - - -
    def HandleXscaleLeftCoarse(self):
        DisplaceXscale(-CoarseInc)
        scene.DrawAll()
    # - - - - - - - - - - - - - - - - - -
    def HandleXscaleLeftFine(self):
        DisplaceXscale(-FineInc)
        scene.DrawAll()
    # - - - - - - - - - - - - - - - - - -
    def HandleXscaleRightFine(self):
        DisplaceXscale(+FineInc)
        scene.DrawAll()
    # - - - - - - - - - - - - - - - - - -
    def HandleXscaleRightCoarse(self):
        DisplaceXscale(+CoarseInc)
        scene.DrawAll()
    # - - - - - - - - - - - - - - - - - -
    # - - - - - - - - - - - - - - - - - -
    def HandleYscaleLeftCoarse(self):
        DisplaceYscale(-CoarseInc)
        scene.DrawAll()
    # - - - - - - - - - - - - - - - - - -
    def HandleYscaleLeftFine(self):
        DisplaceYscale(-FineInc)
        scene.DrawAll()
    # - - - - - - - - - - - - - - - - - -
    def HandleYscaleRightFine(self):
        DisplaceYscale(+FineInc)
        scene.DrawAll()
    # - - - - - - - - - - - - - - - - - -
    def HandleYscaleRightCoarse(self):
        DisplaceYscale(+CoarseInc)
        scene.DrawAll()
    # - - - - - - - - - - - - - - - - - -
    # - - - - - - - - - - - - - - - - - -
    def HandleZscaleLeftCoarse(self):
        DisplaceZscale(-CoarseInc)
        scene.DrawAll()
    # - - - - - - - - - - - - - - - - - -
    def HandleZscaleLeftFine(self):
        DisplaceZscale(-FineInc)
        scene.DrawAll()
    # - - - - - - - - - - - - - - - - - -
    def HandleZscaleRightFine(self):
        DisplaceZscale(+FineInc)
        scene.DrawAll()
    # - - - - - - - - - - - - - - - - - -
    def HandleZscaleRightCoarse(self):
        DisplaceZscale(+CoarseInc)
        scene.DrawAll()
    # - - - - - - - - - - - - - - - - - -

    def Unhide(self):
        # Unhide the major buttons
        self.ShowStatus("Use the buttons below to move groups.")

        self.XtranLeftCoarse.configure(state=NORMAL)
        self.XtranLeftFine.configure(state=NORMAL)
        self.XtranRightFine.configure(state=NORMAL)
        self.XtranRightCoarse.configure(state=NORMAL)

        self.YtranLeftCoarse.configure(state=NORMAL)
        self.YtranLeftFine.configure(state=NORMAL)
        self.YtranRightFine.configure(state=NORMAL)
        self.YtranRightCoarse.configure(state=NORMAL)

        self.ZtranLeftCoarse.configure(state=NORMAL)
        self.ZtranLeftFine.configure(state=NORMAL)
        self.ZtranRightFine.configure(state=NORMAL)
        self.ZtranRightCoarse.configure(state=NORMAL)

        self.BulgeLeftCoarse.configure(state=NORMAL)
        self.BulgeLeftFine.configure(state=NORMAL)
        self.BulgeRightFine.configure(state=NORMAL)
        self.BulgeRightCoarse.configure(state=NORMAL)

        self.XscaleLeftCoarse.configure(state=NORMAL)
        self.XscaleLeftFine.configure(state=NORMAL)
        self.XscaleRightFine.configure(state=NORMAL)
        self.XscaleRightCoarse.configure(state=NORMAL)

        self.YscaleLeftCoarse.configure(state=NORMAL)
        self.YscaleLeftFine.configure(state=NORMAL)
        self.YscaleRightFine.configure(state=NORMAL)
        self.YscaleRightCoarse.configure(state=NORMAL)

        self.ZscaleLeftCoarse.configure(state=NORMAL)
        self.ZscaleLeftFine.configure(state=NORMAL)
        self.ZscaleRightFine.configure(state=NORMAL)
        self.ZscaleRightCoarse.configure(state=NORMAL)

        self.NameEntry.configure(state=NORMAL)
        self.buttonNormGo.configure(state=NORMAL)


    # - - - - - - - - - - - - - - - - - -

    def HandleGroupStart(self):
        # Pick up the group names and make the regular buttons visible.
        global SelectedGroups
        try:
            SelectedIndexes = self.GroupBox.curselection()
            for N in SelectedIndexes:
                intN = int(N)
                SelectedGroups[intN] = self.GroupBox.get(intN)
        except:
            pass  # we already set default to GroupList[0]
        self.Unhide()
        # Set up the real action.
        LocateAllVerts()

    # - - - - - - - - - - - - - - - - - -

    def HandleNormGo(self):
        # Get the name:
        NewName = self.NameEntry.get()
        if not NewName: NewName = 'Explode'
        SpawnResult(NewName)
        self.master.destroy()

    # - - - - - - - - - - - - - - - - - -

    def HandleCancel(self):
        self.master.destroy()

#------------------------------------------------------------
# End defs, begin action here.
#------------------------------------------------------------

InitFirst()
root = Tk()
app = App(root,'')
app.Update()
root.mainloop()

#------------------------------------------------------------
# End TK loop.
#-----------------------------------------------------------------------------
